﻿using Kivii.DataAnnotations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Entities
{
    /// <summary>
    /// 账户类型
    /// </summary>
    public enum AccountType
    {
        /// <summary>
        /// 不支持的账户类型
        /// </summary>
        [Description("不支持的账户类型")]
        Unsupported = 0,

        /// <summary>
        ///现金账户，用于收到的现金，只用于组织内，不用于客户，如业务员收取现金，可转存至银行帐户，备用金
        /// </summary>
        [Description("现金账户")]
        Cash,

        /// <summary>
        ///刷卡账户,用于收到的刷卡,只用于组织内,个人业务员使用 
        /// </summary>
        [Description("刷卡账户")]
        Pos,

        /// <summary>
        ///银行账户，可清零或调帐，不用于客户，只用于组织内
        /// </summary>
        [Description("银行账户")]
        Bank,

        /// <summary>
        /// 支付宝账户
        /// </summary>
        [Description("支付宝账户")]
        AliPay,

        /// <summary>
        /// 微信账户
        /// </summary>
        [Description("微信账户")]
        WeChat,
        /// <summary>
        ///存款账户，专指内部转帐的账户，也可转至Bank，唯一性，资金,关联各种业务的账户，如贷款 
        /// </summary>
        [Description("存款账户")]
        Deposit,

        /// <summary>
        ///优惠券,一次性使用，多不退少补
        /// </summary>
        [Description("优惠券")]
        Coupon,

        /// <summary>
        ///储值卡，只能充值和消费，不能转出
        /// </summary>
        [Description("储值卡")]
        StoredCard,

        /// <summary>
        ///业务专用账户，可设多个业务账户，用于记录不同业务的收入
        /// </summary>
        [Description("业务专用账户")]
        Biz,

        /// <summary>
        /// 平衡系统所有账户金额为0,一个系统仅有一个,即系统与系统外部有金额往来就新增账户明细
        /// </summary>
        [Description("平衡账户")]
        Balance,

        /// <summary>
        /// 折扣账户,各部门仅有一个
        /// </summary>
        [Description("折扣账户")]
        Discount
    }

    [Api(Description = "账户")]
    [Alias(Configs.TableNameAccount)]
    public class Account : EntityWithMetadata, IEntityInAssemblyDb,
        IEntityHasCreator, IEntityHasOrganization
    {
        #region 账户所有人，禁止更新
        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid OwnerKvid { get; set; }

        [ApiMember(Description = "所有者名称")]
        [IgnoreUpdate]
        [StringLength(500), Required]
        public string OwnerName { get; set; }
        #endregion

        #region 帐户相关 账户类型禁止更新

        [ApiMember(Description = "账户名称,如果是Bank类型，可以写银行名称，如果是其它类型，这个应该是账户的友好名称，不应该和OwnerName重复")]
        [StringLength(200)]
        [Required]
        public string Name { get; set; }

        [ApiMember(Description = "账户编号")]
        [StringLength(200), Default("")]
        public string SerialNumber { get; set; }


        [ApiMember(Description = "类型，账户类型")]
        [StringLength(50), Required]
        public AccountType Type { get; set; }

        [ApiMember(Description = "失效日期，仅针对StoredCard储值卡 和 Coupon 优惠券")]
        public DateTime? ExpiredDate { get; set; }
        #endregion

        #region 金额相关，账户单位禁止更新，额度和金额都是内部更新

        [ApiMember(Description = "授信额度，仅针对存款账户")]
        [InternalSetter]
        [DecimalLength(15, 2), Default(0)]
        public decimal CreditLine { get; set; }

        [ApiMember(Description = "当前金额")]
        [InternalSetter]
        [DecimalLength(15, 2), Default(0)]
        public decimal Amount { get; set; }

        [ApiMember(Description = "单位")]
        [IgnoreUpdate]
        [StringLength(50), Required]
        public CurrencyUnit Currency { get; set; }

        #endregion

        #region 文本 摘要 备注
        [ApiMember(Description = "摘要，最大500字")]
        [StringLength(500), Default("")]
        public string Summary { get; set; }

        [ApiMember(Description = "备注，最大2000字")]
        [StringLength(2000), Default("")]
        public string Remark { get; set; }
        #endregion

        #region 操作相关  Creator

        [ApiMember(Description = "创建人Kvid ")]
        [IgnoreUpdate]
        [CurrentMemberKvid]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        [IgnoreUpdate]
        [StringLength(50), CurrentMemberName]
        public string CreatorName { get; set; }


        [ApiMember(Description = "组织机构Id,当组织有子组织时区分所属组织")]
        [IgnoreUpdate]
        [CurrentOrganizationKvid]
        public Guid OrganizationKvid { get; set; }
        #endregion
    }
}
