﻿using Kivii.DataAnnotations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances
{
    /// <summary>
    /// 通用的扩展方法集合
    /// </summary>
    internal static class Extension
    {
        #region 获取枚举类型描述
        /// <summary>
        /// 获取枚举值的描述特性
        /// </summary>
        /// <param name="value">要获取描述的枚举值</param>
        /// <returns>返回枚举值的描述；如果没有描述特性，则返回枚举值的名称</returns>
        /// <example>
        /// <code>
        /// Status status = Status.Pending;
        /// string description = status.GetDescription(); // 返回 "待处理"
        /// </code>
        /// </example>
        public static string GetDescription(this Enum value)
        {
            // 通过反射获取枚举值对应的字段信息
            FieldInfo field = value.GetType().GetField(value.ToString());

            if (field != null)
            {
                // 从字段中获取DescriptionAttribute特性
                DescriptionAttribute attribute = Attribute.GetCustomAttribute(field,
                    typeof(DescriptionAttribute)) as DescriptionAttribute;

                if (attribute != null && !string.IsNullOrEmpty(attribute.Description))
                {
                    // 如果找到了描述特性，则返回其Description属性值
                    return attribute.Description;
                }
            }

            // 如果没有找到描述特性或字段信息为空，则返回枚举值的字符串表示
            return value.ToString();
        }

        /// <summary>
        /// 获取指定枚举类型的所有值和描述，并返回为键值对列表
        /// </summary>
        /// <typeparam name="T">枚举类型</typeparam>
        /// <returns>包含枚举值和描述的键值对列表</returns>
        public static List<KeyValuePair<string, T>> GetAllValuesAndDescriptions<T>() where T : Enum
        {
            // 创建结果列表
            List<KeyValuePair<string, T>> result = new List<KeyValuePair<string, T>>();

            // 获取枚举类型的所有值
            Array enumValues = Enum.GetValues(typeof(T));

            // 遍历所有枚举值
            foreach (T value in enumValues)
            {
                // 获取当前枚举值的描述
                string description = GetDescription((Enum)(object)value);

                // 添加到结果列表
                result.Add(new KeyValuePair<string, T>(description, value));
            }

            return result;
        }
        #endregion

        #region 获取随机字符，用于生成子流水号时如果仍然重复则添加随机字符后缀
        // 全局Random实例，用于生成种子
        private static readonly Random _global = new Random();

        // 线程本地Random实例，每个线程有自己独立的实例
        [ThreadStatic]
        private static Random _local;

        /// <summary>
        /// 线程安全的随机字符串生成方法
        /// </summary>
        /// <param name="length">要生成的字符串长度，默认3</param>
        /// <returns>返回随机字符串，如果length<=0则返回空字符串</returns>
        public static string GetRandomCharsThreadSafe(int length = 3)
        {
            // 参数校验：如果传入长度小于等于0，直接返回空字符串
            if (length <= 0)
            {
                return string.Empty;
            }

            // 定义可选的字符集合（包含大小写字母和数字）
            const string chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";

            // 检查当前线程是否已经初始化了Random实例
            if (_local == null)
            {
                // 加锁确保多线程环境下只有一个线程能进入初始化
                lock (_global)
                {
                    // 再次检查，防止其他线程已经初始化
                    if (_local == null)
                    {
                        // 使用全局Random生成种子，创建线程本地Random实例
                        _local = new Random(_global.Next());
                    }
                }
            }

            // 创建用于存储结果的字符数组
            char[] result = new char[length];

            // 循环生成每个随机字符
            for (int i = 0; i < length; i++)
            {
                // 从chars字符串中随机选取一个字符
                result[i] = chars[_local.Next(chars.Length)];
            }

            // 将字符数组转换为字符串并返回
            return new string(result);
        }
        #endregion
    }
}
